import { unsafe_$ as $, unsafe_mutateSubgraph as mutateSubgraph, unsafe_MutatorFlow as MutatorFlow } from "@typespec/compiler/experimental";
/**
 * Prepares operation for client representation. This includes adding an options bag for optional parameters.
 * @param operation operation to be prepared
 * @returns the prepared operation
 */
export function prepareOperation($, operation) {
  return mutateSubgraph($.program, [httpParamsMutator], operation).type;
}

/**
 * Mutates anonymous types to be named.
 */
const anonymousMutatorRecord = {
  filter(t) {
    return MutatorFlow.MutateAndRecur;
  },
  mutate(t, clone, _program, realm) {
    if (!clone.name) {
      clone.name = $(realm).type.getPlausibleName(clone);
    }
  }
};
export const anonymousMutator = {
  name: "Anonymous types",
  Model: anonymousMutatorRecord,
  Union: anonymousMutatorRecord
};

/**
 * Mutates the operation so that the parameters model is split into required and optional parameters.
 */
export const httpParamsMutator = {
  name: "Http parameters",
  Operation: {
    filter() {
      return MutatorFlow.DoNotRecur;
    },
    mutate(o, clone, _program, realm) {
      const tk = $(realm);
      const httpOperation = tk.httpOperation.get(o);
      const returnType = tk.httpOperation.getReturnType(httpOperation);
      clone.returnType = returnType;
      const params = tk.httpRequest.getParameters(httpOperation, ["query", "header", "path", "body", "contentType"]);
      if (!params) {
        return;
      }
      clone.parameters = params;
      const optionals = [...clone.parameters.properties.values()].filter(p => p.optional).reduce((acc, prop) => {
        acc[prop.name] = prop;
        return acc;
      }, {});
      const optionsBag = tk.model.create({
        properties: optionals
      });
      const optionsProp = tk.modelProperty.create({
        name: "options",
        type: optionsBag,
        optional: true
      });
      for (const [key, prop] of clone.parameters.properties) {
        if (prop.optional || isConstantHeader(tk, prop)) {
          clone.parameters.properties.delete(key);
        }
      }
      if (Object.keys(optionals).length > 0) {
        clone.parameters.properties.set("options", optionsProp);
      }
    }
  }
};
export function isConstantHeader($, modelProperty) {
  if (!$.modelProperty.isHttpHeader(modelProperty)) {
    return false;
  }
  if ("value" in modelProperty.type && modelProperty.type.value !== undefined) {
    return true;
  }
  return false;
}